@@ -7,20 +7,49 @@
 #' @param recursive,all.files options passed to \code{\link{list.files}};
 #'   even if \code{all.files = TRUE}, \code{'.git'} and \code{'.Rproj.user'}
 #'   folders are always discarded
+#' @param trash logical, whether to add a trash to the gadget, allowing to
+#'   restore the files or folders you delete
 #'
 #' @note You can run the gadget for the current directory from the Addins menu
 #'   within RStudio ('Explore current folder').
 #'
 #' @import shiny miniUI
-#' @importFrom rstudioapi getThemeInfo navigateToFile
+#' @importFrom rstudioapi getThemeInfo navigateToFile sendToConsole isAvailable
 #' @importFrom tools file_ext
 #' @importFrom shinyAce aceEditor
 #' @importFrom stats setNames
 #' @importFrom base64enc dataURI
 #' @importFrom utils combn head tail
+#' @importFrom R.utils copyDirectory
 #' @export
+#' @examples library(jsTreeR)
+#'
+#' # copy a folder to a temporary location for the illustration:
+#' tmpDir <- tempdir()
+#' folder <- file.path(tmpDir, "htmlwidgets")
+#' htmlwidgets <- system.file("htmlwidgets", package = "jsTreeR")
+#' R.utils::copyDirectory(htmlwidgets, folder)
+#' # we use a copy because the actions performed in the gadget are
+#' # actually executed on the files system!
+#'
+#' # explore and manipulate the folder (drag-and-drop, right-click):
+#' if(interactive()){
+#'   folderGadget(folder)
+#' }
+#'
+#' # the 'trash' option allows to restore the elements you delete:
+#' if(interactive()){
+#'   folderGadget(folder, trash = TRUE)
+#' }
+#'
+#' # you can open several folders:
+#' folder1 <- file.path(folder, "lib")
+#' folder2 <- file.path(folder, "gadget")
+#' if(interactive()){
+#'   folderGadget(c(folder1, folder2))
+#' }
 folderGadget <- function(
-  dirs = ".", tabs = FALSE, recursive = TRUE, all.files = FALSE
+  dirs = ".", tabs = FALSE, recursive = TRUE, all.files = FALSE, trash = FALSE
 ) {
 
   stopifnot(is.character(dirs))
@@ -34,6 +63,8 @@ folderGadget <- function(
     attachNamespace("jsTreeR")
   }
 
+  rstudio <- isAvailable()
+
   icons <- list(
     dockerfile = "supertinyicon-docker",
     gitignore = "supertinyicon-git",
@@ -47,6 +78,7 @@ folderGadget <- function(
     md = "supertinyicon-markdown",
     markdown = "supertinyicon-markdown",
     rmd = "supertinyicon-markdown",
+    pdf = "supertinyicon-pdf",
     rs = "supertinyicon-rust",
     ru = "supertinyicon-ruby",
     svg = "supertinyicon-svg",
@@ -72,13 +104,21 @@ folderGadget <- function(
       sprintf("  var tree = $(\"#%s\").jstree(true);", treeId),
       "  var items;",
       "  if(node.type === \"file\" || exts.indexOf(node.type) > -1) {",
-      "    items = $.extend(Items(tree, node, false), items_file(tree, node));",
       "    var ext = fileExtension(node.text);",
+      "    items = Items(tree, node, false);",
+      "    if(!isBinary(ext)) {",
+      "      items = $.extend(items, items_file(tree, node));",
+      "    }",
       "    if(isImage(ext)) {",
       "      items = $.extend(items, item_image(tree, node, ext));",
+      "    } else if(ext === 'pdf') {",
+      "      items = $.extend(items, item_pdf(tree, node));",
       "    }",
       "  } else {",
-      "    items = $.extend(item_create(tree, node), Items(tree, node, true));",
+      "    items = $.extend(",
+      "      $.extend(item_create(tree, node), Items(tree, node, true)),",
+      "      item_rerun(tree, node)",
+      "    );",
       "  }",
       "  return items;",
       "}"
@@ -120,7 +160,9 @@ folderGadget <- function(
         )
       }
     }
-    lapply(dat$item[dat$parent == "root"], f)
+    nodes <- lapply(dat$item[dat$parent == "root"], f)
+    nodes[[1L]][["state"]] <- list(opened = TRUE)
+    nodes
   }
 
   readFolder <- function(dir, recursive, all.files){
@@ -191,6 +233,47 @@ folderGadget <- function(
 
   parents <- renameDuplicates(parents)
 
+  if(trash){
+    parents <- c(parents, "_TRASH_")
+    jstrees <- c(jstrees, "trash")
+    trashNodes <- lapply(seq_len(ndirs), function(i){
+      list(
+        text = parents[i],
+        id = paste0("trash-", jstrees[i]),
+        type = "folder",
+        state = list(opened = TRUE)
+      )
+    })
+    grid <- list(
+      columns = list(
+        list(
+          minWidth = 200,
+          header = "Element",
+          headerClass = "bolditalic yellow centered",
+          wideValueClass = "cssclass"
+        ),
+        # list(
+        #   maxWidth = 200,
+        #   value = "location",
+        #   title = "location",
+        #   header = "Location",
+        #   wideValueClass = "cssclass",
+        #   headerClass = "bolditalic yellow centered",
+        #   wideCellClass = "centered ellipsis"
+        # ),
+        list(
+          width = 100,
+          value = "button",
+          header = "Restore?",
+          wideValueClass = "cssclass",
+          headerClass = "bolditalic yellow centered",
+          wideCellClass = "centered"
+        )
+      )
+      #      width = 500
+    )
+  }
+
   toPattern <- function(x){
     gsub(
       "(\\.|\\||\\(|\\)|\\[|\\]|\\{|\\}|\\^|\\$|\\*|\\+|\\?)",
@@ -225,9 +308,11 @@ folderGadget <- function(
 
   types <- append(list(
     file = list(
-      icon = "glyphicon glyphicon-file"
+      icon = "glyphicon glyphicon-file brown"
     ),
-    folder = list()
+    folder = list(
+      icon = "fa fa-folder folder"
+    )
   ), setNames(lapply(names(icons), function(ext){
     list(icon = icons[[ext]])
   }), names(icons)))
@@ -235,7 +320,7 @@ folderGadget <- function(
   checkCallback <- JS(
     "function(operation, node, parent, position, more) {",
     "  if(operation === 'move_node') {",
-    "    if(parent.id === '#' || parent.type === 'file') {",
+    "    if(parent.id === '#' || parent.type !== 'folder') {",
     "      return false;",
     "    }",
     "  }",
@@ -243,7 +328,7 @@ folderGadget <- function(
     "}"
   )
 
-  themeInfo <- getThemeInfo()
+  theme <- ifelse(rstudio, getThemeInfo()[["editor"]], "cobalt")
 
   if(ndirs != 2L){
     tabs <- ndirs >= 3L
@@ -263,268 +348,43 @@ folderGadget <- function(
       webp = "image/webp"
     )
 
+  www <- function(file){
+    system.file("htmlwidgets", "gadget", file, package = "jsTreeR")
+  }
+
   ui <- miniPage(
 
     tags$head(
-      tags$style(
-        HTML(
-          c(
-            ".jstree-proton {font-weight: bold;}",
-            ".jstree-anchor {font-size: medium;}",
-            "input[id$='-search'] {background-color: seashell;}",
-            "#shiny-modal .modal-dialog div[class^='modal-'] {",
-            "  background-color: maroon;",
-            "}",
-            ".gadget-block-button {background-color: transparent;}",
-            ".gadget-tabs-container ul.gadget-tabs>li.active>a {",
-            "  font-weight: bold;",
-            "  background-color: paleturquoise;",
-            "}",
-            ".gadget-tabs-container ul.gadget-tabs>li:not(.active)>a {",
-            "  background-color: rgba(175,238,238,0.5);",
-            "}",
-            ".ace_scrollbar::-webkit-scrollbar-track {",
-            "  border-radius: 10px;",
-            "  background-color: crimson;",
-            "}",
-            ".ace_scrollbar::-webkit-scrollbar {",
-            "  background-color: transparent;",
-            "}",
-            ".ace_scrollbar::-webkit-scrollbar-thumb {",
-            "  border-radius: 10px;",
-            "  background-color: tomato;",
-            "}"
-          )
+      includeScript(www("pdfobject.min.js")),
+      includeCSS(www("gadget.css")),
+      if(rstudio){
+        tags$script(HTML("var rstudio = true;"))
+      }else{
+        tags$script(HTML("var rstudio = false;"))
+      },
+      if(trash){
+        tagList(
+          tags$script(HTML("var Trash = true;")),
+          includeCSS(www("trash.css")),
+          includeScript(www("trash.js"))
         )
-      ),
+      }else{
+        tags$script(HTML("var Trash = false;"))
+      },
       tags$script(
         HTML(
-          "var copiedNode = null;",
           sprintf("var exts = [%s];", toString(paste0("'", names(icons), "'"))),
           sprintf("var sep = \"%s\";", .Platform$file.sep),
           sprintf(
             "var imageExts = [%s];",
             toString(paste0("'", names(imageMIME), "'"))
-          ),
-          "function fileExtension(fileName) {",
-          "  if(/\\./.test(fileName)) {",
-          "    var splitted = fileName.split('.');",
-          "    return splitted[splitted.length - 1].toLowerCase();",
-          "  } else {",
-          "    return null;",
-          "  }",
-          "}",
-          "function isImage(ext) {",
-          "  return imageExts.indexOf(ext) > -1;",
-          "}",
-          "function item_image(tree, node, ext) {",
-          "  return {",
-          "    Image: {",
-          "      separator_before: true,",
-          "      separator_after: true,",
-          "      label: \"View image\",",
-          "      action: function(obj) {",
-          "        Shiny.setInputValue(",
-          "          'viewImage',",
-          "          {",
-          "            instance: tree.element.attr('id'),",
-          "            path: tree.get_path(node, sep),",
-          "            ext: ext",
-          "          }",
-          "        );",
-          "      }",
-          "    }",
-          "  };",
-          "}",
-          "function Items(tree, node, paste) {",
-          "  return {",
-          "    Copy: {",
-          "      separator_before: true,",
-          "      separator_after: false,",
-          "      label: \"Copy\",",
-          "      action: function(obj) {",
-          "        tree.copy(node);",
-          "        copiedNode = {node: node, operation: 'copy'};", # use get_buffer and clear_buffer instead ?
-          "      }",
-          "    },",
-          "    Cut: {",
-          "      separator_before: false,",
-          "      separator_after: !paste,",
-          "      label: \"Cut\",",
-          "      action: function(obj) {",
-          "        tree.cut(node);",
-          "        copiedNode = {instance: tree, node: node, operation: 'cut'};",
-          "      }",
-          "    },",
-          "    Paste: paste ? {",
-          "      separator_before: false,",
-          "      separator_after: true,",
-          "      label: \"Paste\",",
-          "      _disabled: copiedNode === null,",
-          "      action: function(obj) {",
-          "        var children = tree.get_node(node).children.map(",
-          "          function(child) {return tree.get_text(child);}",
-          "        );",
-          "        if(children.indexOf(copiedNode.node.text) === -1) {",
-          #"          tree.paste(node);",
-          "          var operation = copiedNode.operation;",
-          "          tree.copy_node(copiedNode.node, node, 0, function() {",
-          "            Shiny.setInputValue('operation', operation);",
-          "            setTimeout(function() {",
-          "              Shiny.setInputValue('operation', 'rename');",
-          "            }, 0);",
-          "          });",
-          "          if(operation === 'cut') {",
-          "            copiedNode.instance.delete_node(copiedNode.node);",
-          "          }",
-          "          copiedNode = null;",
-          "        }",
-          "      }",
-          "    } : null,",
-          "    Rename: {",
-          "      separator_before: true,",
-          "      separator_after: false,",
-          "      label: \"Rename\",",
-          "      action: function(obj) {",
-          "        tree.edit(node, null, function() {",
-          "          var nodeType = tree.get_type(node);",
-          "          if(nodeType === 'file' || exts.indexOf(nodeType) > -1) {",
-          "            var nodeText = tree.get_text(node);",
-          "            var ext = fileExtension(nodeText);",
-          "            if(ext !== null) {",
-          "              if(exts.indexOf(ext) > -1) {",
-          "                tree.set_type(node, ext);",
-          "              } else {",
-          "                tree.set_type(node, 'file');",
-          "              }",
-          "            }",
-          "          }",
-          "        });",
-          "      }",
-          "    },",
-          "    Remove: {",
-          "      separator_before: false,",
-          "      separator_after: true,",
-          "      label: \"Remove\",",
-          "      action: function(obj) {",
-          "        tree.delete_node(node);",
-          "      }",
-          "    }",
-          "  };",
-          "}",
-          "function items_file(tree, node) {",
-          "  return {",
-          "    Open: {",
-          "      separator_before: true,",
-          "      separator_after: false,",
-          "      label: \"Open in RStudio\",",
-          "      action: function(obj) {",
-          "        Shiny.setInputValue(",
-          "          'openFile',",
-          "          {",
-          "            instance: tree.element.attr('id'),",
-          "            path: tree.get_path(node, sep)",
-          "          }",
-          "        );",
-          "      }",
-          "    },",
-          "    Edit: {",
-          "      separator_before: false,",
-          "      separator_after: true,",
-          "      label: \"Edit\",",
-          "      action: function(obj) {",
-          "        Shiny.setInputValue(",
-          "          'editFile',",
-          "          {",
-          "            instance: tree.element.attr('id'),",
-          "            path: tree.get_path(node, sep)",
-          "          },",
-          "          {priority: 'event'}",
-          "        );",
-          "      }",
-          "    }",
-          "  };",
-          "}",
-          "function item_create(tree, node) {",
-          "  return {",
-          "    Create: {",
-          "      separator_before: true,",
-          "      separator_after: true,",
-          "      label: \"Create\",",
-          "      action: false,",
-          "      submenu: {",
-          "        File: {",
-          "          separator_before: true,",
-          "          separator_after: false,",
-          "          label: \"File\",",
-          "          action: function(obj) {",
-          "            var children = tree.get_node(node).children.map(",
-          "              function(child) {return tree.get_text(child);}",
-          "            );",
-          "            node = tree.create_node(node, {type: \"file\"});",
-          "            tree.edit(",
-          "              node, null, function() {",
-          "                var nodeText = tree.get_text(node);",
-          "                if(children.indexOf(nodeText) > -1) {",
-          "                  tree.delete_node(node);",
-          "                } else {",
-          "                  if(/\\./.test(nodeText)) {",
-          "                    var splitted = nodeText.split('.');",
-          "                    var ext = splitted[splitted.length - 1]",
-          "                                .toLowerCase();",
-          "                    if(exts.indexOf(ext) > -1) {",
-          "                      tree.set_type(node, ext);",
-          "                    }",
-          "                  }",
-          "                  Shiny.setInputValue(",
-          "                    'createdNode',",
-          "                    {",
-          "                      instance: tree.element.attr('id'),",
-          "                      type: 'file',",
-          "                      path: tree.get_path(node, sep)",
-          "                    }",
-          "                  );",
-          "                }",
-          "              }",
-          "            );",
-          "          }",
-          "        },",
-          "        Folder: {",
-          "          separator_before: false,",
-          "          separator_after: true,",
-          "          label: \"Folder\",",
-          "          action: function(obj) {",
-          "            var children = tree.get_node(node).children.map(",
-          "              function(child) {return tree.get_text(child);}",
-          "            );",
-          "            node = tree.create_node(node, {type: \"folder\"});",
-          "            tree.edit(",
-          "              node, null, function() {",
-          "                if(children.indexOf(tree.get_text(node)) > -1) {",
-          "                  tree.delete_node(node);",
-          "                } else {",
-          "                  Shiny.setInputValue(",
-          "                    'createdNode',",
-          "                    {",
-          "                      instance: tree.element.attr('id'),",
-          "                      type: 'folder',",
-          "                      path: tree.get_path(node, sep)",
-          "                    }",
-          "                  );",
-          "                }",
-          "              }",
-          "            );",
-          "          }",
-          "        }",
-          "      }",
-          "    }",
-          "  };",
-          "}"
+          )
         )
-      )
+      ),
+      includeScript(www("gadget.js"))
     ),
 
-    if(tabs){
+    if(tabs || trash){
       do.call(function(...){
         miniTabstripPanel(
           ...,
@@ -533,14 +393,41 @@ folderGadget <- function(
             border = NULL
           )
         )
-      }, lapply(seq_len(ndirs), function(i){
-        miniTabPanel(
-          parents[i],
-          miniContentPanel(
-            jstreeOutput(jstrees[i])
+      }, if(tabs || ndirs != 2L){
+        lapply(seq_len(ndirs + trash), function(i){
+          miniTabPanel(
+            if(ndirs > 1L) parents[i],
+            icon = if(ndirs == 1L) icon(ifelse(i == 1L, "folder-open", "trash")),
+            miniContentPanel(
+              jstreeOutput(jstrees[i])
+            )
           )
+        })
+      } else {
+        list(
+          miniTabPanel(
+            NULL,
+            icon = icon("folder-open"),
+            miniContentPanel(
+              do.call(
+                fillRow,
+                lapply(jstrees[c(1L,2L)], function(id){
+                  jstreeOutput(id)
+                })
+              )
+            )
+          ),
+          # if(trash){
+            miniTabPanel(
+              NULL,
+              icon = icon("trash"),
+              miniContentPanel(
+                jstreeOutput("trash")
+              )
+            )
+          # }
         )
-      }))
+      })
     }else{
       miniContentPanel(
         miniButtonBlock(
@@ -563,12 +450,50 @@ folderGadget <- function(
   )
 
 
-  server <- function(input, output){
+  TMPDIR <- tempdir()
+
+  server <- function(input, output, session){
 
     observeEvent(input[["done"]], {
       stopApp()
     })
 
+    observeEvent(input[["rerun"]], {
+      path <- file.path(
+        paths[input[["rerun"]][["instance"]]],
+        input[["rerun"]][["path"]]
+      )
+      session$onSessionEnded(function(){
+        if(rstudio){
+          code <- sprintf("jsTreeR::folderGadget('%s', trash = %s)", path, trash)
+          sendToConsole(code)
+        }else{
+          jsTreeR::folderGadget(path, trash = trash)
+        }
+      })
+      stopApp()
+    })
+
+    observeEvent(input[["viewPDF"]], {
+      filePath <- file.path(
+        paths[input[["viewPDF"]][["instance"]]],
+        input[["viewPDF"]][["path"]]
+      )
+      b64 <- dataURI(file = filePath, mime = "application/pdf")
+      script <- HTML(
+        sprintf("var b64 = '%s';", b64),
+        "PDFObject.embed(b64, '#pdf');"
+      )
+      showModal(modalDialog(
+        tagList(
+          tags$div(id = "pdf", style = "height: 70vh;"),
+          tags$script(script)
+        ),
+        easyClose = TRUE,
+        size = "l"
+      ))
+    })
+
     observeEvent(input[["viewImage"]], {
       filePath <- file.path(
         paths[input[["viewImage"]][["instance"]]],
@@ -654,7 +579,7 @@ folderGadget <- function(
           "aceEditor",
           value = paste0(suppressWarnings(readLines(filePath)), collapse = "\n"),
           mode = ifelse(is.null(mode), "plain_text", mode),
-          theme = gsub(" ", "_", tolower(themeInfo[["editor"]])),
+          theme = gsub(" ", "_", tolower(theme)),
           tabSize = 2,
           height = "60vh"
         ),
@@ -685,17 +610,48 @@ folderGadget <- function(
     })
 
     observeEvent(input[["jsTreeDeleted"]], {
-      path <- file.path(
-        paths[input[["jsTreeDeleted"]][["instance"]]],
+      instance <- input[["jsTreeDeleted"]][["instance"]]
+      pathTail <-
         paste0(input[["jsTreeDeleted"]][["path"]], collapse = .Platform$file.sep)
-      )
-      if(file.exists(path)){
-        unlink(path, recursive = TRUE)
+      if(instance != "trash"){
+        path <- file.path(paths[instance], pathTail)
+        if(file.exists(path)){
+          if(trash){
+            tmpPath <- file.path(TMPDIR, instance, pathTail)
+            if(!dir.exists(d <- dirname(tmpPath))){
+              dir.create(d, recursive = TRUE)
+            }
+            if(dir.exists(path)){
+              copyDirectory(path, tmpPath)
+            }else{
+              file.copy(path, tmpPath)
+            }
+          }
+          unlink(path, recursive = TRUE)
+        }
       }
     })
 
+    observeEvent(input[["restore"]], {
+      path <- file.path(
+        paths[input[["restore"]][["instance"]]],
+        input[["restore"]][["path"]]
+      )
+      tmpPath <- file.path(
+        TMPDIR,
+        input[["restore"]][["instance"]],
+        input[["restore"]][["path"]]
+      )
+      if(!dir.exists(d <- dirname(path))) dir.create(d, recursive = TRUE)
+      if(dir.exists(tmpPath)){
+        copyDirectory(tmpPath, path)
+      }else{
+        file.copy(tmpPath, path)
+      }
+      unlink(tmpPath, recursive = TRUE)
+    })
+
     observeEvent(input[["createdNode"]], {
-      print(input[["createdNode"]])
       nodePath <- file.path(
         paths[input[["createdNode"]][["instance"]]],
         input[["createdNode"]][["path"]]
@@ -758,30 +714,53 @@ folderGadget <- function(
     for(treeId in jstrees){
       local({
         id <- treeId
-        output[[id]] <- renderJstree({
-          jstree(
-            nodes[[id]],
-            types = types,
-            dragAndDrop = TRUE,
-            checkboxes = FALSE,
-            multiple = FALSE,
-            theme = "proton",
-            contextMenu = list(select_node = FALSE, items = js(id)),
-            checkCallback = checkCallback,
-            sort = TRUE,
-            search = list(
-              show_only_matches = TRUE,
-              case_sensitive = FALSE,
-              search_leaves_only = FALSE
+        if(id != "trash"){
+          output[[id]] <- renderJstree({
+            jstree(
+              nodes[[id]],
+              types = types,
+              dragAndDrop = TRUE,
+              checkboxes = FALSE,
+              multiple = FALSE,
+              theme = "proton",
+              contextMenu = list(select_node = FALSE, items = js(id)),
+              checkCallback = checkCallback,
+              sort = TRUE,
+              search = list(
+                show_only_matches = TRUE,
+                case_sensitive = FALSE,
+                search_leaves_only = FALSE
+              )
             )
-          )
-        })
+          })
+        }else{
+          output[["trash"]] <- renderJstree({
+            jstree(
+              trashNodes,
+              types = types,
+              dragAndDrop = FALSE,
+              checkboxes = FALSE,
+              multiple = FALSE,
+              theme = "default",
+              contextMenu = FALSE,
+              checkCallback = TRUE,
+              sort = FALSE,
+              search = FALSE,
+              grid = grid
+            )
+          })
+          outputOptions(output, "trash", suspendWhenHidden = FALSE)
+        }
       })
     }
 
   }
 
-  runGadget(shinyApp(ui, server), stopOnCancel = FALSE)
+  runGadget(
+    shinyApp(ui, server),
+    stopOnCancel = FALSE,
+    viewer = if(rstudio) paneViewer() else browserViewer()
+  )
 
 }
 
